local hp = BeneCast.Helper;

local type, pairs, next = type, pairs, next;

-- *****************************************************************************
-- Helper functions
-- *****************************************************************************

-- Recycling functions
-- Swiped from AceGUIWidget-TreeGroup, updated to use wipe() and clear metatables
do
	local pool = setmetatable({},{__mode='k'})
	hp.new = function()
		local t = next(pool)
		if t then
			pool[t] = nil
			return t
		else
			return {}
		end
	end
	hp.del = function(t)
		wipe(t)
		setmetatable(t, nil)
		pool[t] = true
		return nil
	end
end

-- Function to clear the table given as the parameter
hp._ClearTable = function(t, recursively)

	if t ~= nil and type(t) == 'table' then
		if not recursively then
			wipe(t)
		else
			-- Removes every element of the table starting at the end
			for k, v in pairs(t) do
				if type(v) == 'table' then
					hp._ClearTable(v, true);
					t[k] = hp.del(v);
				else
					t[k] = nil;
				end
			end
		end
		return t;
	else
		return {};
	end
	
end

hp._CopyTable = function(t, recursive)
	if t ~= nil then
		local new = hp.new();
		for k, v in pairs(t) do
			if (type(v) == 'table') and recursive then
				new[k] = hp._CopyTable(v,true);
			else
				new[k] = v;
			end
		end
		return new;
	else
		return nil;
	end
end

hp._CopyTableToExisting = function(s, d, recursive)
	if s == nil or d == nil or type(s) ~= 'table' or type(d) ~= 'table' then
		return;
	end
	for k, v in pairs(s) do
		if (type(v) == 'table') and recursive then
			d[k] = hp._CopyTable(v,true);
		else
			d[k] = v;
		end
	end
	return d;
end

-- Function to set a lot of attributes in one time
hp._SetManyAttributes = function(self, ...)
	for i=1,select("#", ...),2 do
		local att,val = select(i, ...);
		if not att then
			return;
		end
		self:SetAttribute(att,val);
	end
end

-- Function to get the key of the table member who's value = val
hp._GetKeyForVal = function(t, val)
	for k, v in pairs(t) do
		if v == val then
			return k;
		end
	end
end


-- Function to get the key of the table member that has a certain table member with value = val
hp._GetKeyForValInMember = function(t, m, val)
	for k, v in pairs(t) do
		if type(v) == 'table' and v[m] == val then
			return k;
		end
	end
end

-- Function to set the value of the table member who's value = oldval
-- 'count' determines how many set's may be done, nil = infinite
hp._ReplaceValues = function(t, oldval, newval, count)
	for k, v in pairs(t) do
		if v == oldval then
			t[k] = newval;
			if count ~= nil then
				count = count - 1;
				if count <= 0 then
					do break end;
				end
			end
		end
	end
end

-- Function to set the value of a table member's member that has another/the same member with value = oldval
-- 'count' determines how many set's may be done, nil = infinite
hp._ReplaceValuesByMember = function(t, oldmember, oldval, newmember, newval, count)
	for k, v in pairs(t) do
		if type(v) == 'table' and v[oldmember] == oldval then
			v[newmember] = newval;
			if count ~= nil then
				count = count - 1;
				if count <= 0 then
					do break end;
				end
			end
		end
	end
end

-- Version of table.getn that gets a count of ALL members
hp._CountMembers = function(t)

	local cnt = 0;
	for k, v in pairs(t) do
		cnt = cnt + 1;
	end
	return cnt;

end

-- Like the isnull function in SQL, returns a substitute value if first value is nil
hp._isnil = function(value, repl)
	if value == nil then
		return repl;
	else
		return value;
	end
end